/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#ifndef _yoocom_HXX
#define _yoocom_HXX

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "list.hxx"

#define yyout yoocom_yyout
#define yyin yoocom_yyin 

extern FILE *yyout, *yyin ;

#define SIZEOF_TOKEN 256
#define VALUE 0
#define REF 1
#define PTR 2 

#define CPP 0
#define ASL 1
#define CLIENT 0
#define SERVER 1 
#define CONNECTED 0
#define DISCONNECTED 1 


class SvcClass ;
typedef enum { T_CHAR = 0, T_INT = 1, T_DOUBLE = 2, T_STRING = 3,
	       T_UCHAR = 4, T_UINT = 5, T_SHORT = 6, T_USHORT = 7,
	       T_FLOAT = 8, T_VOID = 9, T_OTHER = 10
	     } TypeId ;	   


class SvcType {
  protected :
  char* typename ;
  TypeId id ;

  public :
  SvcType(SvcType* t) : typename(t->typename), id(t->id) {}
  SvcType(char* n, TypeId tid) : typename(strdup(n)), id (tid) {}
  void genAslType(char* s) ;
  TypeId tid() { return id ; }
  virtual void genType(char* s) {} ;
  virtual int isflow() { return 0 ; }
  virtual int istable() { return 0 ; }
  virtual int istemplate() { return 0 ; }
  virtual int issimple() { return 0 ; }
} ;

class SvcSimple : public SvcType {
  public :
  SvcSimple(SvcType* t) : SvcType(t) {}
  void genType(char* s) { fprintf(yyout, "%s%s", s, this->typename) ; }
  int issimple() { return 1 ; }
} ;

class SvcFlow : public SvcType {
  public :
  SvcFlow(SvcType* t) : SvcType(t) {}
  void genType(char* s) { fprintf(yyout, "%sFlow<%s>", s, this->typename) ; }
  int isflow() { return 1 ; }
} ;

class SvcTable : public SvcType {
  public :
  SvcTable(SvcType* t) : SvcType(t) {}
  void genType(char* s) { fprintf(yyout, "%sTable<%s>", s, this->typename) ; }
  int istable() { return 1 ; }
} ;

class SvcTemplate : public SvcType {
  private :
  char* templ ;
  
  public :
  SvcTemplate(char* n, SvcType* t) : SvcType(t), templ(strdup(n)) {}
  void genType(char* s) { fprintf(yyout, "%s%s<%s>", s, templ, typename) ; }
  int istemplate() { return 1 ; }
} ;

class SvcParam {
  private :
  int direction ;
  char* name ;
  SvcType* typ ;

  public :
  SvcParam(int, SvcType*, char* name = NULL) ;
  int in() ;
  int out() ;
  TypeId tid() { return typ->tid() ; }
  void genName(char* s) { fprintf(yyout, "%s%s", s, this->name) ; }
  void genType(char* s) { typ->genType(s);  }
  void genAslType(char* pref) { typ->genAslType(pref) ; }
  int isflow() { return typ->isflow() ; }
  int istable() { return typ->istable() ; }
  int istemplate() { return typ->istemplate() ; }
  int issimple() { return typ->issimple() ; }
} ;

class SvcMember {
  protected :
  char* name ;

  public :
  virtual int isLocal() = 0 ;
} ;

class SvcRemoteMember : public SvcMember {
  private :
  SvcType* result ;
  ListPtr<SvcParam> parameter_list ;
    
  public :
  SvcRemoteMember(SvcType* r, char* n, ListPtr<SvcParam>* p_l) ;
  int isLocal() ;
  void cppGenClient(int numproc, char* n, int mode) ;
  void cppGenServer(int numproc, char* cl) ;
  void cppGenRegister(int numproc, char* cl) ;
  void aslGenServer(int num, char* cl) ;
  void aslGenClient(int num, char* cl) ;
} ;

class SvcLocalMember : public SvcMember {
  protected :
  SvcType* type ;
  
  public :
  SvcLocalMember(SvcType* r, char* n) ;
  int isLocal() ;
  void genName(char* pref) { fprintf(yyout, "%s%s", pref, name) ; }
  virtual void genAslType(char* pref) { type->genAslType(pref) ; }
} ;

class SvcMultipleLocalMember : public SvcLocalMember {
  private :
  int size ;

  public :
  SvcMultipleLocalMember(SvcType* r, char* n, int s) ;
  void genAslType(char* pref) {
    fprintf(yyout, "%s", pref) ;
    type->genAslType("DeclarerTable ") ;
    fprintf(yyout, " %d", size) ;
  }
} ;

struct SvcHead {
  char* name ;
  SvcClass* base ;

  SvcHead(char* n, SvcClass* b = NULL) : name(strdup(n)), base(b) {} ;
  void cppGenCtor(char* b, char* pref = "") ;
} ;


class SvcStruct {
  private :
  char* name ;
  ListPtr<SvcLocalMember>  attribut_list ;

  public :
  SvcStruct(char* n, ListPtr<SvcMember>*) ;
  void aslGen() ;
} ;


class SvcClass {
  friend SvcClass* findClass(char* n) ;

  private :
  SvcHead* head ;
  ListPtr<SvcLocalMember>  attribut_list ;
  ListPtr<SvcRemoteMember> method_list ;
  
  public :
  SvcClass(SvcHead* h, ListPtr<SvcMember>* m_l); 
  void cppGenServerRemote() ;
  void cppGenClient() ;
  void cppGenCtor(char* b) ;
  void aslGenServer() ;
  void aslGenClient() ;
  int numproc() {
    return (head->base==NULL?0:head->base->numproc()) + method_list.count();}
  char* classname() { return this->head->name ; }
} ;


class SvcIdent {
  private :

  int _size ;
  char* _name ;

  public :

  SvcIdent(char* n, int s) : _size(s), _name(strdup(n)) {} ;
  char* name() { return _name ; }
  int size() { return _size ; }
} ;

void cppGenServerHeader() ;
SvcClass* findClass(char*) ;
void cppGenClient() ;
void aslGenClient() ;
void aslGenServer() ;

EXTERN ListPtr<SvcClass> SchemaClass ;
EXTERN ListPtr<SvcStruct> SchemaStruct ;
EXTERN int linecount ;
EXTERN int bracket_count ;

// Selection variables for command line options
EXTERN_INIT (int lang_gen, -1) ;
EXTERN_INIT (int side_gen, -1) ;
EXTERN_INIT (int conn_gen, -1) ;



#endif
